;//*****************(ASM) COPYRIGHT 2023 Masses-Chip ****************************
;* Project Name		  : M8P632_ADC.prj
;* File Name		  : M8P632_ADC.ASM
;* Author			  : MASSES CHIP
;* Version           		 : V1.05
;* Date              		 : 2024/03/05
;* Web				  : www.masses-chip.com
;* FAE				  : Luo
;					  : QQ：411680975
;//******************************************************************************
;
;Option:16M/16T,WDT-Enable-288ms,RST-IO,BOR 2.4V 
;说明：本范例AD采集通道为AIN0(IOA0),时钟源选择为Fcpu/16T，内部参考2V
;采集10次AD值，去掉最大值和最小值，对剩下的AD值求平均,结果存放在R_ADC_sum_H、
;R_ADC_sum_L中，作为一次AD值
;//******************************************************************************
;//==============================================================================
;//***********************************头文件*************************************
;//用户不可更改
;//==============================================================================
#INCLUDE	 	ZC.INC

;//==============================================================================
;//******************************定义全局变量************************************
;//用户可以自行看需求定义变量
;//==============================================================================
userdata	udata	H'00'
R_Count			res    1
R_ADC_max_L		res    1
R_ADC_max_H		res    1
R_ADC_min_L		res    1
R_ADC_min_H		res    1
R_ADC_sum_L		res    1
R_ADC_sum_H		res    1

;//==============================================================================
;//*********************************C O D E**************************************
;//==============================================================================	
		ORG			0000H
		GOTO		MainProgram
		ORG			0008H
		GOTO		Interrupt	

;//==============================================================================
;//********************************程序主函数************************************
;//==============================================================================			
;//--------------------------------程序初始化------------------------------------
MainProgram:
		CALL		SYSTEM_INIT				;//系统初始化
		CALL		Clr_RAM_00H_FFH			;//清零RAM
		CALL		ADC_INIT				;//ADC初始化
;//------------------------------主函数循环--------------------------------------		
MainLoop:
		CLRWDT
		CALL		ADC_Work
		GOTO		MainLoop

;//==============================================================================
;//******************************中断服务子程序**********************************
;//==============================================================================
;//中断进来
Interrupt:
		PUSH								;//压栈（保存A，STATUS）
		NOP
Interrupt_END:
;//中断结束
		POP									;//压栈（恢复A，STATUS）
	    RETIE

;//==============================================================================
;//*********************************ADC初始化************************************
;//ADC设置：选择通道为AIN0，数据格式为高位在前，
;//时钟源选择为Fcpu/16T，内部参考2V
;//==============================================================================	
ADC_INIT:
;//1、端口设置
		BSET		ANSA,0
		BCLR		OEA,0
;//2、ADC设置
		MOVIA		0x90
		MOVAR		ADCON0
		MOVIA		0x05
		MOVAR		ADCON1
		MOVIA		0x00
		MOVAR		ADCON2
		RETURN

;//==============================================================================
;//********************************系统初始化************************************
;//系统时钟初始化：快钟、慢钟都正常工作，不进休眠
;//IO初始化:设置IO状态为输出低，且关闭上下拉
;//中断总开关：关闭总开关
;//==============================================================================		
SYSTEM_INIT:
;//-----------------------------------------------------------------------           
;//Init Clock
		MOVIA       b'00000000'
		MOVAR		OSCM
;//Init IO	
		MOVIA       b'00000000'
		MOVAR		OSCM
;//Init IO
        ;//IOA
        MOVIA      	0x00 
		MOVAR		ANSA		;//0:数字口	  1:模拟口 
        MOVAR     	PUA			;//0:上拉关闭 1:上拉使能  	 	
        MOVAR      	PDA			;//0:下拉关闭 1:下拉使能
		MOVAR      	IOA        	;//0:输出低   1:输出高
		MOVIA      	0xFF    	
        MOVAR      	OEA			;//0:输入     1:输出   	
		;//IOB
		MOVIA		0x00
        MOVAR      	PUB
        MOVAR      	PDB
        MOVAR      	IOB
		MOVIA      	0xFF
        MOVAR      	OEB
		;//IOC
        MOVIA      	0x00
		MOVAR		ANSC
        MOVAR      	PUC
        MOVAR      	PDC
        MOVAR      	IOC
		MOVIA      	0xFF
        MOVAR      	OEC
;//------------------------------------------------------------------------
;//Init Interrupt
		BCLR		OPTION,GIE		;//1:使能总中断  0:屏蔽总中断
		RETURN

;//==============================================================================
;//*******************************RAM清零(00H~FFH)*******************************
;//RAM清零：RAM区00H~FFH全部清零
;//==============================================================================				
Clr_RAM_00H_FFH:
		CLRR    	FSR0
ClrRam_Loop:
		MOVIA		0x00
		MOVAR		INDF0
		INCR		FSR0,R
		MOVIA		0xFF
		JCMPAR		FSR0
		GOTO		ClrRam_Loop
		MOVIA     	0x00
		MOVAR		INDF0
RAM_INIT_END:
		RETURN

;//==============================================================================
;//*********************************ADC工作**************************************
;//采集10次值除去最大最小，剩余的求平均值
;//==============================================================================	
#include	ADC_Work.INC
		END

;//******************************END OF FILE*************************************