/*******************(C) COPYRIGHT 2023 Masses-Chip ****************************
* Project Name       : M8P632_PWM0.prj
* File Name          : M8P632_PWM0.C
* Author             : MASSES CHIP
* Version            : V1.05
* Date               : 2024/03/05
* Web    			 : www.masses-chip.com
* FAE				 : Luo
*  				 	 : QQ：411680975
*******************************************************************************
*Option:16M/16T,WDT-Enable-288ms,RST-IO,BOR 2.4V 
*说明：本范例以PWM0P(IOB0)、PWM0N(IOB1）输出1KHz波形，输出控制每定时100ms调节一次
;IO口波形使能位控制输出的波形的开关，前、后死区时间32us
******************************************************************************/
//=============================================================================
//*******************************头文件和调用申明******************************
//=============================================================================
//用户不可更改
#include "ZC.h"
#include "MassesChip_Define.h"

//-----------------------------------------------------------------------------
//用户可以自行更改
void SYSTEM_INIT(void);					//系统初始化
void Clr_RAM_00H_FFH(void);				//寄存器清零
void PWM0_INIT(void);					//PWM0初始化
void Delay_100ms(void);					//延时100ms

//===============================================================================
//******************************中断特殊函数保护*********************************
/* 如果主程序和中断都用到特殊函数时（例：有变量的乘除法、传参和数组等需要读取或
写入通用指针指向的字节操作），那么在进中断前需要对编译器的临时性寄存器保存。 */
//===============================================================================
#define			Save_Special_Function		1		//中断使用特殊函数时必须置“1”

//如果主程序和中断都用到数组、指针、查表时，需对所用寄存器保存
#define			Save_Fsr					1		//中断使用寻址时必须置“1”

//===============================================================================
//**********************************定义变量*************************************
//用户可以自行看需求定义变量
//===============================================================================
#ifdef	Save_Special_Function
u8 R_Save_Stk00,R_Save_Stk01,R_Save_Stk02,R_Save_Stk03,R_Save_Stk04,R_Save_Stk05;
u8 R_Save_Stk06,R_Save_Stk07,R_Save_Stk08,R_Save_Stk09,R_Save_Stk10,R_Save_Stk11;
#endif
 
#ifdef	Save_Fsr
u8 R_Save_Fsr0;
u8 R_Save_Fsr1;
u8 R_Save_Hbuf;
#endif

//=============================================================================
//********************************程序主函数***********************************
//=============================================================================
void main(void)					  		//程序主函数
{
	SYSTEM_INIT();						//系统初始化
	Clr_RAM_00H_FFH();					//清零RAM
	PWM0_INIT();						//PWM0初始化
	while(1)
	{
		__CLRWDT();						//清除看门狗
		PWM0POE = 1;
		PWM0NOE = 1;
		Delay_100ms();
		PWM0POE = 0;
		PWM0NOE = 0;
		Delay_100ms();
	}
}

//=============================================================================
//******************************中断服务子程序*********************************
//高优先级中断入口需调用空函数做断点保护
//注：中断处理调用子函数处理，不要再这里直接处理
//=============================================================================
void int_isr(void) __interrupt
{
	__PUSH();                    	//进栈（保存A，STATUS）

//---------------------------------------
// 备份关键寄存器，若中断中确保没用到，可删除对应代码，这样可节约寄存器
#ifdef	Save_Special_Function	 							//保存中断中使用库中特殊函数产生的临时寄存器
__asm;
	MOVR		STK00,0
	MOVAR		_R_Save_Stk00
	MOVR		STK01,0
	MOVAR		_R_Save_Stk01
	MOVR		STK02,0
	MOVAR		_R_Save_Stk02
	MOVR		STK03,0
	MOVAR		_R_Save_Stk03
	MOVR		STK04,0
	MOVAR		_R_Save_Stk04
	MOVR		STK05,0
	MOVAR		_R_Save_Stk05
	MOVR		STK06,0
	MOVAR		_R_Save_Stk06
	MOVR		STK07,0
	MOVAR		_R_Save_Stk07
	MOVR		STK08,0
	MOVAR		_R_Save_Stk08
	MOVR		STK09,0
	MOVAR		_R_Save_Stk09
	MOVR		STK10,0
	MOVAR		_R_Save_Stk10
	MOVR		STK11,0
	MOVAR		_R_Save_Stk11
__endasm;
#endif

#ifdef	Save_Fsr	 										//保存中断寻址指针FSR0、FSR1、HBUF
	R_Save_Fsr0 = FSR0;
	R_Save_Fsr1 = FSR1;
	R_Save_Hbuf = HBUF;
#endif
//---------------------------------------

//中断处理	
	__NOP();

//---------------------------------------
// 备份关键寄存器，若中断中确保没用到，可删除对应代码，这样可节约寄存器
#ifdef	Save_Special_Function	 							//恢复中断中使用库中特殊函数产生的临时寄存器
__asm;
	MOVR		_R_Save_Stk00,0
	MOVAR		STK00
	MOVR		_R_Save_Stk01,0
	MOVAR		STK01
	MOVR		_R_Save_Stk02,0
	MOVAR		STK02
	MOVR		_R_Save_Stk03,0
	MOVAR		STK03
	MOVR		_R_Save_Stk04,0
	MOVAR		STK04
	MOVR		_R_Save_Stk05,0
	MOVAR		STK05
	MOVR		_R_Save_Stk06,0
	MOVAR		STK06
	MOVR		_R_Save_Stk07,0
	MOVAR		STK07
	MOVR		_R_Save_Stk08,0
	MOVAR		STK08
	MOVR		_R_Save_Stk09,0
	MOVAR		STK09
	MOVR		_R_Save_Stk10,0
	MOVAR		STK10
	MOVR		_R_Save_Stk11,0
	MOVAR		STK11
__endasm;
#endif

#ifdef	Save_Fsr	 										//恢复中断寻址指针FSR0、FSR1、HBUF
	FSR0 = R_Save_Fsr0;
	FSR1 = R_Save_Fsr1;
	HBUF = R_Save_Hbuf;	
#endif
//---------------------------------------

	__POP();                    	//出栈（恢复A，STATUS）
}

//=============================================================================
//*********************************PWM0初始化**********************************
//TC0设置：递增模式，16M，高频系统时钟，1:64分频	t1 = 计数/（时钟/分频）= 1ms
//PWM0设置：时基选择TC0，端口输出PWM0P、PWM0N波形，占空比50%	频率 = 1/t1 =1KHz
//死区设置：前、后死区时间为 32us
//=============================================================================
void PWM0_INIT(void)
{
//1、TC0设置
	T0CR = 0x0E;
	TC0CL = 0;
	TC0CH = 0;
	TC0PRL = 249;
	TC0PRH = 0;
	TC0EN = 1;
//2、PWM0设置
	PWM0CR = 0x1C;
	PWMS = 0x00;
	PWM0DL = (249+1)/2;
	PWM0DH = 0;
	PWMDEADT = 0x77;
	PWM0EN = 1;
}

//延时100ms
void Delay_100ms(void)
{
	unsigned int R_DELAY_COUNT_i;
	unsigned int R_DELAY_COUNT_j;
	for(R_DELAY_COUNT_i=50;R_DELAY_COUNT_i>0;R_DELAY_COUNT_i--)
	{
		for(R_DELAY_COUNT_j=110;R_DELAY_COUNT_j>0;R_DELAY_COUNT_j--);
	}
}

//=============================================================================
//*********************************系统初始化**********************************
//系统时钟初始化：快钟、慢钟都正常工作，不进休眠
//IO初始化:设置IO状态为输出低，且关闭上下拉
//中断总开关：关闭总开关
//=============================================================================	
void SYSTEM_INIT(void)
{
//----------------------------------------------------
//Init Clock
	OSCM = 0x00;
//----------------------------------------------------	            
//Init IO
	//IOA
	ANSA = 0x00;               //0:数字口	1:模拟口 
	PUA = 0x00;                //0:上拉关闭 1:上拉使能
	PDA = 0x00;				   //0:下拉关闭 1:下拉使能
	IOA = 0x00;                //IO口数据
	OEA = 0xFF;                //0:输入 1:输出
	//IOB
	PUB = 0x00;
	PDB = 0x00;
	IOB = 0x00;
	OEB = 0xFF;
	//IOC
	ANSC = 0x00;
	PUC = 0x00;
	PDC = 0x00;
	IOC = 0x00;
	OEC = 0xFF;
//----------------------------------------------------     
//Init Interrupt
	GIE = 0;					//1:使能总中断  0:屏蔽总中断
}

//=============================================================================
//********************************RAM清零(00H~FFH)*****************************
//RAM清零：RAM区00H~FFH全部清零
//=============================================================================
void Clr_RAM_00H_FFH(void)
{
	for(FSR0=0;FSR0!=0xFF;FSR0++)
	{
		INDF0 = 0x00;
	}
	if(FSR0=0xFF)
	{
		INDF0 = 0x00;
	}
}

//*********************************END OF FILE**********************************