;//*****************(ASM) COPYRIGHT 2023 Masses-Chip ****************************
;* Project Name		  : M8P632_TC2G.prj
;* File Name		  : M8P632_TC2G.ASM
;* Author			  : MASSES CHIP
;* Version           		 : V1.05
;* Date              		 : 2024/03/05
;* Web				  : www.masses-chip.com
;* FAE				  : Luo
;					  : QQ：411680975
;//******************************************************************************
;Option:16M/16T,WDT-Enable-288ms,RST-IO,BOR 2.4V 
;说明：本范例是TC2以TC0(1ms定时)门控模式为上升沿到上升沿
;当TC0第二次上升沿TC2G产生中断(2ms)并翻转IOB0口
;//******************************************************************************
;//==============================================================================
;//***********************************头文件*************************************
;//用户不可更改
;//==============================================================================
#INCLUDE	 	ZC.INC

;//==============================================================================
;//******************************定义全局变量************************************
;//用户可以自行看需求定义变量
;//==============================================================================
userdata	udata	H'00'
R_Flag		res    1
#define	b_TC2G_F		R_Flag,0			;//TC2G中断标志

;//==============================================================================
;//*********************************C O D E**************************************
;//==============================================================================	
		ORG			0000H
		GOTO		MainProgram
		ORG			0008H
		GOTO		Interrupt

;//==============================================================================
;//********************************程序主函数************************************
;//==============================================================================			
;//--------------------------------程序初始化------------------------------------
MainProgram:
		CALL       	SYSTEM_INIT				;//系统初始化
		CALL       	Clr_RAM_00H_FFH			;//清零RAM
		CALL       	TC2G_INIT				;//TC2G初始化
;//------------------------------主函数循环--------------------------------------		
MainLoop:
		CLRWDT
		JBTS1		b_TC2G_F
		GOTO		MainLoop
		BCLR		b_TC2G_F
		JBTS1		IOB,0
		GOTO		IOB0_SET_1
IOB0_SET_0:
		BCLR		IOB,0
		GOTO		MainLoop
IOB0_SET_1:
		BSET		IOB,0
		GOTO		MainLoop

;//==============================================================================
;//******************************中断服务子程序**********************************
;//门控定时2ms，进一次中断，设置一次TC2G中断标志位，启动门控
;//==============================================================================
;//中断进来
Interrupt:
		PUSH								;//压栈（保存A，STATUS）
		JBTS1		INTF0,TC2GIF
		GOTO		Interrupt_END
		BCLR		INTF0,TC2GIF
		BSET		b_TC2G_F
		BSET		T2GCR,6
Interrupt_END:
;//中断结束
		POP									;//压栈（恢复A，STATUS）
	    RETIE

;//==============================================================================
;//*********************************TC2G初始化***********************************
;//TC0设置：8模式，16M，高频系统时钟，1:64分频		t=(249+1)/(16/64)=1ms
;//TC2设置：递增模式，16M，高频系统时钟，1:64分频
;//TC2G设置：TC2门控源TC0溢出周期，门控模式上升沿到上升沿
;//开启TC2G溢出中断和总中断
;//==============================================================================	
TC2G_INIT:
;//1、门控源TC0设置
		MOVIA      	0x0E
		MOVAR	    T0CR
		CLRR		TC0CL
		CLRR	   	TC0CH
		MOVIA      	249
		MOVAR	   	TC0PRL
		MOVIA		0
		MOVAR		TC0PRH
;//2、TC2设置
		MOVIA      	0x0E
		MOVAR	    T2CR
		CLRR		TC2CL
		CLRR	   	TC2CH
		MOVIA      	249
		MOVAR	   	TC2PRL
		MOVIA		0
		MOVAR		TC2PRH
;//3、TC2G门控设置
		MOVIA		0x82
		MOVAR		T2GCR
		BSET		T0CR,7
		BSET       	T2CR,7
		BSET		T2GCR,6
;//4、中断设置
		BCLR		INTF0,TC2GIF
		BSET		INTCR0,TC2GIE
		BSET       	OPTION,7
		RETURN

;//==============================================================================
;//**********************************系统初始化**********************************
;//系统时钟初始化：快钟、慢钟都正常工作，不进休眠
;//IO初始化:设置IO状态为输出低，且关闭上下拉
;//中断总开关：关闭总开关
;//==============================================================================		
SYSTEM_INIT:
;//-----------------------------------------------------------------------           
;//Init Clock
		MOVIA       b'00000000'
		MOVAR		OSCM
;//Init IO
       ;//IOA
        MOVIA      	0x00 
		MOVAR		ANSA		;//0:数字口	  1:模拟口 
        MOVAR     	PUA			;//0:上拉关闭 1:上拉使能  	 	
        MOVAR      	PDA			;//0:下拉关闭 1:下拉使能
		MOVAR      	IOA        	;//0:输出低   1:输出高
		MOVIA      	0xFF    	
        MOVAR      	OEA			;//0:输入     1:输出   	
		;//IOB
		MOVIA		0x00
        MOVAR      	PUB
        MOVAR      	PDB
        MOVAR      	IOB
		MOVIA      	0xFF
        MOVAR      	OEB
		;//IOC
        MOVIA      	0x00
		MOVAR		ANSC
        MOVAR      	PUC
        MOVAR      	PDC
        MOVAR      	IOC
		MOVIA      	0xFF
        MOVAR      	OEC
;//------------------------------------------------------------------------      
;//Init Interrupt
       	BCLR		OPTION,GIE		;//1:使能总中断  0:屏蔽总中断
		RETURN
		
;//==============================================================================
;//*****************************RAM清零(00H~FFH)*****************************
;//RAM清零：RAM区00H~FFH全部清零
;//==============================================================================				
Clr_RAM_00H_FFH:
		CLRR    	FSR0
ClrRam_Loop:
		MOVIA		0x00
		MOVAR		INDF0
		INCR		FSR0,R
		MOVIA		0xFF
		JCMPAR		FSR0
		GOTO		ClrRam_Loop
		MOVIA     	0x00			
		MOVAR		INDF0
RAM_INIT_END:
		RETURN
		END

;//******************************END OF FILE************************************